@@ -26,10 +26,10 @@
 #include <gtk/gtk.h>
 #include <gnome-keyring.h>
 
-#include "seahorse-gkr-module.h"
+#include "seahorse-gkr.h"
+#include "seahorse-gkr-keyring.h"
 
 #include "seahorse-object.h"
-#include "seahorse-source.h"
 
 typedef enum {
     SEAHORSE_GKR_USE_OTHER,
@@ -63,10 +63,14 @@ struct _SeahorseGkrItemClass {
 
 GType                        seahorse_gkr_item_get_type           (void);
 
-SeahorseGkrItem*             seahorse_gkr_item_new                (SeahorseSource *source,
+SeahorseGkrItem *            seahorse_gkr_item_new                (SeahorseGkrKeyring *keyring,
                                                                    const gchar *keyring_name,
                                                                    guint32 item_id);
 
+void                         seahorse_gkr_item_refresh            (SeahorseGkrItem *self);
+
+void                         seahorse_gkr_item_realize            (SeahorseGkrItem *self);
+
 guint32                      seahorse_gkr_item_get_item_id        (SeahorseGkrItem *self);
 
 const gchar*                 seahorse_gkr_item_get_keyring_name   (SeahorseGkrItem *self);
@@ -91,9 +95,6 @@ const gchar*                 seahorse_gkr_item_get_attribute      (SeahorseGkrIt
 const gchar* 		     seahorse_gkr_find_string_attribute   (GnomeKeyringAttributeList *attrs, 
              		                                           const gchar *name);
 
-GQuark                       seahorse_gkr_item_get_cannonical     (const gchar *keyring_name, 
-                                                                   guint32 item_id);
-
 SeahorseGkrUse               seahorse_gkr_item_get_use            (SeahorseGkrItem *self);
 
 #endif /* __SEAHORSE_GKR_ITEM_H__ */
