@@ -23,37 +23,66 @@
 #include "config.h"
 
 #include "seahorse-gkr.h"
+#include "seahorse-gkr-backend.h"
+#include "seahorse-gkr-dialogs.h"
+#include "seahorse-gkr-keyring-deleter.h"
 #include "seahorse-gkr-keyring.h"
 #include "seahorse-gkr-operation.h"
+#include "seahorse-gkr-actions.h"
 
+#include "seahorse-action.h"
+#include "seahorse-deletable.h"
+#include "seahorse-lockable.h"
 #include "seahorse-progress.h"
 #include "seahorse-util.h"
+#include "seahorse-viewable.h"
+
+#include <gcr/gcr.h>
 
 #include <glib/gi18n.h>
 
 enum {
 	PROP_0,
-	PROP_SOURCE_TAG,
-	PROP_SOURCE_LOCATION,
+	PROP_DESCRIPTION,
 	PROP_KEYRING_NAME,
 	PROP_KEYRING_INFO,
-	PROP_IS_DEFAULT
+	PROP_IS_DEFAULT,
+	PROP_URI,
+	PROP_ACTIONS,
+	PROP_LOCKED,
+	PROP_LOCKABLE,
+	PROP_UNLOCKABLE
 };
 
 struct _SeahorseGkrKeyringPrivate {
+	GHashTable *items;
 	gchar *keyring_name;
 	gboolean is_default;
-	
+	gboolean locked;
+
 	gpointer req_info;
 	GnomeKeyringInfo *keyring_info;
 };
 
-static void seahorse_source_iface (SeahorseSourceIface *iface);
+static void     seahorse_gkr_keyring_place_iface    (SeahorsePlaceIface *iface);
 
-G_DEFINE_TYPE_EXTENDED (SeahorseGkrKeyring, seahorse_gkr_keyring, SEAHORSE_TYPE_OBJECT, 0,
-                        G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_SOURCE, seahorse_source_iface));
+static void     seahorse_keyring_collection_iface   (GcrCollectionIface *iface);
 
-GType
+static void     seahorse_keyring_deletable_iface    (SeahorseDeletableIface *iface);
+
+static void     seahorse_keyring_lockable_iface     (SeahorseLockableIface *iface);
+
+static void     seahorse_keyring_viewable_iface     (SeahorseViewableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (SeahorseGkrKeyring, seahorse_gkr_keyring, SEAHORSE_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (GCR_TYPE_COLLECTION, seahorse_keyring_collection_iface);
+                         G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_PLACE, seahorse_gkr_keyring_place_iface);
+                         G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_DELETABLE, seahorse_keyring_deletable_iface);
+                         G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_LOCKABLE, seahorse_keyring_lockable_iface);
+                         G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_VIEWABLE, seahorse_keyring_viewable_iface);
+);
+
+static GType
 boxed_type_keyring_info (void)
 {
 	static GType type = 0;
@@ -64,10 +93,6 @@ boxed_type_keyring_info (void)
 	return type;
 }
 
-/* -----------------------------------------------------------------------------
- * INTERNAL 
- */
-
 static void
 received_keyring_info (GnomeKeyringResult result, GnomeKeyringInfo *info, gpointer data)
 {
@@ -115,30 +140,44 @@ require_keyring_info (SeahorseGkrKeyring *self)
 void
 seahorse_gkr_keyring_realize (SeahorseGkrKeyring *self)
 {
-	gchar *name, *markup;
-	
-	name = g_strdup_printf (_("Passwords: %s"), self->pv->keyring_name);
-	markup = g_markup_printf_escaped (_("<b>Passwords:</b> %s"), self->pv->keyring_name);
-	
+	const gchar *name;
+	GIcon *icon;
+
+	if (self->pv->keyring_name && g_str_equal (self->pv->keyring_name, "login"))
+		name = _("Login keyring");
+	else
+		name = self->pv->keyring_name;
+
+	icon = g_themed_icon_new ("folder");
+
 	g_object_set (self,
 	              "label", name,
-	              "markup", markup,
 	              "nickname", self->pv->keyring_name,
 	              "identifier", "",
-	              "flags", SEAHORSE_FLAG_DELETABLE,
-	              "icon", "folder",
+	              "flags", SEAHORSE_FLAG_DELETABLE | SEAHORSE_FLAG_PERSONAL,
+	              "icon", icon,
 	              "usage", SEAHORSE_USAGE_OTHER,
 	              NULL);
-	
-	g_free (name);
-	g_free (markup);
+
+	g_object_unref (icon);
 }
 
 void
-seahorse_gkr_keyring_refresh (SeahorseGkrKeyring *self)
+seahorse_gkr_keyring_remove_item (SeahorseGkrKeyring *self,
+                                  guint32 item_id)
 {
-	if (self->pv->keyring_info)
-		load_keyring_info (self);
+	SeahorseGkrItem *item;
+
+	g_return_if_fail (SEAHORSE_IS_GKR_KEYRING (self));
+
+	item = g_hash_table_lookup (self->pv->items, GUINT_TO_POINTER (item_id));
+	if (item != NULL) {
+		g_object_ref (item);
+		g_object_set (item, "place", NULL, NULL);
+		g_hash_table_remove (self->pv->items, GUINT_TO_POINTER (item_id));
+		gcr_collection_emit_removed (GCR_COLLECTION (self), G_OBJECT (item));
+		g_object_unref (item);
+	}
 }
 
 typedef struct {
@@ -164,23 +203,13 @@ keyring_load_free (gpointer data)
 
 /* Remove the given key from the context */
 static void
-remove_key_from_context (gpointer kt, SeahorseObject *dummy, SeahorseSource *sksrc)
+remove_key_from_context (gpointer kt,
+                         gpointer value,
+                         gpointer user_data)
 {
 	/* This function gets called as a GHFunc on the self->checks hashtable. */
-	GQuark keyid = GPOINTER_TO_UINT (kt);
-	SeahorseObject *sobj;
-
-	sobj = seahorse_context_get_object (SCTX_APP (), sksrc, keyid);
-	if (sobj != NULL)
-		seahorse_context_remove_object (SCTX_APP (), sobj);
-}
-
-static void
-insert_id_hashtable (SeahorseObject *object, gpointer user_data)
-{
-	g_hash_table_insert ((GHashTable*)user_data,
-	                     GUINT_TO_POINTER (seahorse_object_get_id (object)),
-	                     GUINT_TO_POINTER (TRUE));
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (user_data);
+	seahorse_gkr_keyring_remove_item (self, GPOINTER_TO_UINT (kt));
 }
 
 static void
@@ -190,13 +219,12 @@ on_keyring_load_list_item_ids (GnomeKeyringResult result,
 {
 	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
 	keyring_load_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
-	SeahorseObjectPredicate pred;
-	SeahorseGkrItem *git;
+	SeahorseGkrItem *item;
 	const gchar *keyring_name;
 	GError *error = NULL;
 	GHashTable *checks;
-	guint32 item_id;
-	GQuark id;
+	GHashTableIter iter;
+	gpointer key;
 
 	closure->request = NULL;
 
@@ -210,33 +238,28 @@ on_keyring_load_list_item_ids (GnomeKeyringResult result,
 	g_return_if_fail (keyring_name);
 
 	/* When loading new keys prepare a list of current */
-	checks = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, NULL);
-	seahorse_object_predicate_clear (&pred);
-	pred.source = SEAHORSE_SOURCE (closure->keyring);
-	pred.type = SEAHORSE_TYPE_GKR_ITEM;
-	seahorse_context_for_objects_full (SCTX_APP (), &pred, insert_id_hashtable, checks);
+	checks = g_hash_table_new (g_direct_hash, g_direct_equal);
+	g_hash_table_iter_init (&iter, closure->keyring->pv->items);
+	while (g_hash_table_iter_next (&iter, &key, NULL))
+		g_hash_table_insert (checks, key, key);
 
 	while (list) {
-		item_id = GPOINTER_TO_UINT (list->data);
-		id = seahorse_gkr_item_get_cannonical (keyring_name, item_id);
+		item = g_hash_table_lookup (closure->keyring->pv->items, list->data);
 
-		git = SEAHORSE_GKR_ITEM (seahorse_context_get_object (seahorse_context_instance (),
-		                                                      SEAHORSE_SOURCE (closure->keyring), id));
-		if (!git) {
-			git = seahorse_gkr_item_new (SEAHORSE_SOURCE (closure->keyring), keyring_name, item_id);
+		if (item == NULL) {
+			item = seahorse_gkr_item_new (closure->keyring, keyring_name,
+			                              GPOINTER_TO_UINT (list->data));
 
 			/* Add to context */
-			seahorse_object_set_parent (SEAHORSE_OBJECT (git), SEAHORSE_OBJECT (closure->keyring));
-			seahorse_context_take_object (seahorse_context_instance (), SEAHORSE_OBJECT (git));
+			g_hash_table_insert (closure->keyring->pv->items, list->data, item);
+			gcr_collection_emit_added (GCR_COLLECTION (closure->keyring), G_OBJECT (item));
 		}
 
-		g_hash_table_remove (checks, GUINT_TO_POINTER (id));
+		g_hash_table_remove (checks, list->data);
 		list = g_list_next (list);
 	}
 
-	g_hash_table_foreach (checks, (GHFunc)remove_key_from_context,
-	                      SEAHORSE_SOURCE (closure->keyring));
-
+	g_hash_table_foreach (checks, (GHFunc)remove_key_from_context, closure->keyring);
 	g_hash_table_destroy (checks);
 
 	seahorse_progress_end (closure->cancellable, res);
@@ -286,18 +309,14 @@ on_keyring_load_cancelled (GCancellable *cancellable,
 }
 
 static void
-seahorse_gkr_keyring_load_async (SeahorseSource *source,
-                                 GCancellable *cancellable,
-                                 GAsyncReadyCallback callback,
-                                 gpointer user_data)
+seahorse_gkr_keyring_load_async (SeahorsePlace *place,
+                                    GCancellable *cancellable,
+                                    GAsyncReadyCallback callback,
+                                    gpointer user_data)
 {
-	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (source);
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (place);
 	keyring_load_closure *closure;
 	GSimpleAsyncResult *res;
-#if TODO
-/* cancellable = g_cancellable_new ();
-g_cancellable_cancel (cancellable); */
-#endif
 
 	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
 	                                 seahorse_gkr_keyring_load_async);
@@ -321,11 +340,11 @@ g_cancellable_cancel (cancellable); */
 }
 
 static gboolean
-seahorse_gkr_keyring_load_finish (SeahorseSource *source,
-                                  GAsyncResult *result,
-                                  GError **error)
+seahorse_gkr_keyring_load_finish (SeahorsePlace *place,
+                                     GAsyncResult *result,
+                                     GError **error)
 {
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (source),
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (place),
 	                      seahorse_gkr_keyring_load_async), FALSE);
 
 	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
@@ -334,52 +353,42 @@ seahorse_gkr_keyring_load_finish (SeahorseSource *source,
 	return TRUE;
 }
 
-static GObject* 
-seahorse_gkr_keyring_constructor (GType type, guint n_props, GObjectConstructParam *props) 
+static void
+seahorse_gkr_keyring_constructed (GObject *obj)
 {
-	GObject *obj = G_OBJECT_CLASS (seahorse_gkr_keyring_parent_class)->constructor (type, n_props, props);
-	SeahorseGkrKeyring *self = NULL;
-	gchar *id;
-	
-	if (obj) {
-		self = SEAHORSE_GKR_KEYRING (obj);
-		
-		g_return_val_if_fail (self->pv->keyring_name, obj);
-		id = g_strdup_printf ("%s:%s", SEAHORSE_GKR_TYPE_STR, self->pv->keyring_name);
-		g_object_set (self, 
-		              "id", g_quark_from_string (id), 
-		              "usage", SEAHORSE_USAGE_NONE, 
-		              NULL);
-		g_free (id);
-	}
-	
-	return obj;
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (obj);
+
+	G_OBJECT_CLASS (seahorse_gkr_keyring_parent_class)->constructed (obj);
+
+	g_return_if_fail (self->pv->keyring_name);
 }
 
 static void
 seahorse_gkr_keyring_init (SeahorseGkrKeyring *self)
 {
 	self->pv = G_TYPE_INSTANCE_GET_PRIVATE (self, SEAHORSE_TYPE_GKR_KEYRING, SeahorseGkrKeyringPrivate);
-	g_object_set (self, "tag", SEAHORSE_GKR_TYPE, "location", SEAHORSE_LOCATION_LOCAL, NULL);
+	self->pv->items = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_object_unref);
 }
 
 static void
 seahorse_gkr_keyring_finalize (GObject *obj)
 {
 	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (obj);
-	
+
+	g_hash_table_destroy (self->pv->items);
+
 	if (self->pv->keyring_info)
 		gnome_keyring_info_free (self->pv->keyring_info);
 	self->pv->keyring_info = NULL;
 	g_assert (self->pv->req_info == NULL);
-    
+
 	g_free (self->pv->keyring_name);
 	self->pv->keyring_name = NULL;
 	
 	if (self->pv->keyring_info) 
 		gnome_keyring_info_free (self->pv->keyring_info);
 	self->pv->keyring_info = NULL;
-	
+
 	G_OBJECT_CLASS (seahorse_gkr_keyring_parent_class)->finalize (obj);
 }
 
@@ -412,13 +421,11 @@ seahorse_gkr_keyring_get_property (GObject *obj, guint prop_id, GValue *value,
                            GParamSpec *pspec)
 {
 	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (obj);
-	
+	gchar *text;
+
 	switch (prop_id) {
-	case PROP_SOURCE_TAG:
-		g_value_set_uint (value, SEAHORSE_GKR_TYPE);
-		break;
-	case PROP_SOURCE_LOCATION:
-		g_value_set_enum (value, SEAHORSE_LOCATION_LOCAL);
+	case PROP_DESCRIPTION:
+		g_value_set_string (value, seahorse_gkr_keyring_get_description (self));
 		break;
 	case PROP_KEYRING_NAME:
 		g_value_set_string (value, seahorse_gkr_keyring_get_name (self));
@@ -429,6 +436,22 @@ seahorse_gkr_keyring_get_property (GObject *obj, guint prop_id, GValue *value,
 	case PROP_IS_DEFAULT:
 		g_value_set_boolean (value, seahorse_gkr_keyring_get_is_default (self));
 		break;
+	case PROP_URI:
+		text = g_strdup_printf ("secret-service://%s", self->pv->keyring_name);
+		g_value_take_string (value, text);
+		break;
+	case PROP_ACTIONS:
+		g_value_take_object (value, seahorse_gkr_keyring_actions_new (self));
+		break;
+	case PROP_LOCKED:
+		g_value_set_boolean (value, seahorse_gkr_keyring_get_locked (self));
+		break;
+	case PROP_LOCKABLE:
+		g_value_set_boolean (value, !seahorse_gkr_keyring_get_locked (self));
+		break;
+	case PROP_UNLOCKABLE:
+		g_value_set_boolean (value, seahorse_gkr_keyring_get_locked (self));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
 		break;
@@ -440,16 +463,16 @@ seahorse_gkr_keyring_class_init (SeahorseGkrKeyringClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-	seahorse_gkr_keyring_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (SeahorseGkrKeyringPrivate));
 
-	gobject_class->constructor = seahorse_gkr_keyring_constructor;
+	gobject_class->constructed = seahorse_gkr_keyring_constructed;
 	gobject_class->finalize = seahorse_gkr_keyring_finalize;
 	gobject_class->set_property = seahorse_gkr_keyring_set_property;
 	gobject_class->get_property = seahorse_gkr_keyring_get_property;
 
-	g_object_class_override_property (gobject_class, PROP_SOURCE_TAG, "source-tag");
-	g_object_class_override_property (gobject_class, PROP_SOURCE_LOCATION, "source-location");
+	g_object_class_override_property (gobject_class, PROP_DESCRIPTION, "description");
+	g_object_class_override_property (gobject_class, PROP_URI, "uri");
+	g_object_class_override_property (gobject_class, PROP_ACTIONS, "actions");
 
 	g_object_class_install_property (gobject_class, PROP_KEYRING_NAME,
 	           g_param_spec_string ("keyring-name", "Gnome Keyring Name", "Name of keyring.", 
@@ -458,22 +481,179 @@ seahorse_gkr_keyring_class_init (SeahorseGkrKeyringClass *klass)
 	g_object_class_install_property (gobject_class, PROP_KEYRING_INFO,
 	           g_param_spec_boxed ("keyring-info", "Gnome Keyring Info", "Info about keyring.", 
 	                               boxed_type_keyring_info (), G_PARAM_READWRITE));
-	
+
 	g_object_class_install_property (gobject_class, PROP_IS_DEFAULT,
 	           g_param_spec_boolean ("is-default", "Is default", "Is the default keyring.",
 	                                 FALSE, G_PARAM_READWRITE));
+
+	g_object_class_install_property (gobject_class, PROP_LOCKED,
+	           g_param_spec_boolean ("locked", "Locked", "Keyring is locked?",
+	                                 FALSE, G_PARAM_READABLE));
+
+	g_object_class_override_property (gobject_class, PROP_LOCKABLE, "lockable");
+	g_object_class_override_property (gobject_class, PROP_UNLOCKABLE, "unlockable");
 }
 
-static void 
-seahorse_source_iface (SeahorseSourceIface *iface)
+static void
+seahorse_gkr_keyring_place_iface (SeahorsePlaceIface *iface)
 {
 	iface->load_async = seahorse_gkr_keyring_load_async;
 	iface->load_finish = seahorse_gkr_keyring_load_finish;
 }
 
-/* -----------------------------------------------------------------------------
- * PUBLIC 
- */
+static guint
+seahorse_gkr_keyring_get_length (GcrCollection *collection)
+{
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (collection);
+	return g_hash_table_size (self->pv->items);
+}
+
+static GList *
+seahorse_gkr_keyring_get_objects (GcrCollection *collection)
+{
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (collection);
+	return g_hash_table_get_values (self->pv->items);
+}
+
+static gboolean
+seahorse_gkr_keyring_contains (GcrCollection *collection,
+                               GObject *object)
+{
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (collection);
+	guint32 item_id;
+
+	if (!SEAHORSE_IS_GKR_ITEM (object))
+		return FALSE;
+	item_id = seahorse_gkr_item_get_item_id (SEAHORSE_GKR_ITEM (object));
+	return g_hash_table_lookup (self->pv->items, GUINT_TO_POINTER (item_id)) ? TRUE : FALSE;
+}
+
+static void
+seahorse_keyring_collection_iface (GcrCollectionIface *iface)
+{
+	iface->get_length = seahorse_gkr_keyring_get_length;
+	iface->get_objects = seahorse_gkr_keyring_get_objects;
+	iface->contains = seahorse_gkr_keyring_contains;
+}
+
+static SeahorseDeleter *
+seahorse_gkr_keyring_create_deleter (SeahorseDeletable *deletable)
+{
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (deletable);
+	return seahorse_gkr_keyring_deleter_new (self);
+}
+
+static void
+seahorse_keyring_deletable_iface (SeahorseDeletableIface *iface)
+{
+	iface->create_deleter = seahorse_gkr_keyring_create_deleter;
+}
+
+static void
+on_keyring_xlock_done (GnomeKeyringResult result,
+                       gpointer user_data)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (g_async_result_get_source_object (user_data));
+	GError *error = NULL;
+
+	if (seahorse_gkr_propagate_error (result, &error))
+		g_simple_async_result_take_error (res, error);
+
+	seahorse_gkr_keyring_load_async (SEAHORSE_PLACE (self), NULL, NULL, NULL);
+	g_simple_async_result_complete_in_idle (res);
+
+	g_object_unref (self);
+}
+
+static void
+seahorse_gkr_keyring_lock_async (SeahorseLockable *lockable,
+                                 GTlsInteraction *interaction,
+                                 GCancellable *cancellable,
+                                 GAsyncReadyCallback callback,
+                                 gpointer user_data)
+{
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (lockable);
+	GSimpleAsyncResult *res;
+
+	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
+	                                 seahorse_gkr_keyring_lock_async);
+
+	gnome_keyring_lock (seahorse_gkr_keyring_get_name (self),
+	                    on_keyring_xlock_done, g_object_ref (res), g_object_unref);
+
+	g_object_unref (res);
+}
+
+static gboolean
+seahorse_gkr_keyring_lock_finish (SeahorseLockable *lockable,
+                                  GAsyncResult *result,
+                                  GError **error)
+{
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (lockable),
+	                      seahorse_gkr_keyring_lock_async), FALSE);
+
+	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+		return FALSE;
+
+	return TRUE;
+}
+
+static void
+seahorse_gkr_keyring_unlock_async (SeahorseLockable *lockable,
+                                   GTlsInteraction *interaction,
+                                   GCancellable *cancellable,
+                                   GAsyncReadyCallback callback,
+                                   gpointer user_data)
+{
+	SeahorseGkrKeyring *self = SEAHORSE_GKR_KEYRING (lockable);
+	GSimpleAsyncResult *res;
+
+	res = g_simple_async_result_new (G_OBJECT (lockable), callback, user_data,
+	                                 seahorse_gkr_keyring_unlock_async);
+
+	gnome_keyring_unlock (seahorse_gkr_keyring_get_name (self), NULL,
+	                      on_keyring_xlock_done, g_object_ref (res), g_object_unref);
+
+	g_object_unref (res);
+}
+
+static gboolean
+seahorse_gkr_keyring_unlock_finish (SeahorseLockable *lockable,
+                                    GAsyncResult *result,
+                                    GError **error)
+{
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (lockable),
+	                      seahorse_gkr_keyring_unlock_async), FALSE);
+
+	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+		return FALSE;
+
+	return TRUE;
+}
+
+static void
+seahorse_keyring_lockable_iface (SeahorseLockableIface *iface)
+{
+	iface->lock_async = seahorse_gkr_keyring_lock_async;
+	iface->lock_finish = seahorse_gkr_keyring_lock_finish;
+	iface->unlock_async = seahorse_gkr_keyring_unlock_async;
+	iface->unlock_finish = seahorse_gkr_keyring_unlock_finish;
+}
+
+static void
+seahorse_gkr_keyring_show_viewer (SeahorseViewable *viewable,
+                                  GtkWindow *parent)
+{
+	seahorse_gkr_keyring_properties_show (SEAHORSE_GKR_KEYRING (viewable),
+	                                      parent);
+}
+
+static void
+seahorse_keyring_viewable_iface (SeahorseViewableIface *iface)
+{
+	iface->show_viewer = seahorse_gkr_keyring_show_viewer;
+}
 
 SeahorseGkrKeyring*
 seahorse_gkr_keyring_new (const gchar *keyring_name)
@@ -488,6 +668,15 @@ seahorse_gkr_keyring_get_name (SeahorseGkrKeyring *self)
 	return self->pv->keyring_name;
 }
 
+const gchar *
+seahorse_gkr_keyring_get_description (SeahorseGkrKeyring *self)
+{
+	g_return_val_if_fail (SEAHORSE_IS_GKR_KEYRING (self), NULL);
+	if (g_strcmp0 (self->pv->keyring_name, "login") == 0)
+		return _("A keyring that is automatically unlocked on login");
+	return _("A keyring used to store passwords");
+}
+
 GnomeKeyringInfo*
 seahorse_gkr_keyring_get_info (SeahorseGkrKeyring *self)
 {
@@ -514,6 +703,7 @@ seahorse_gkr_keyring_set_info (SeahorseGkrKeyring *self, GnomeKeyringInfo *info)
 	g_object_freeze_notify (obj);
 	seahorse_gkr_keyring_realize (self);
 	g_object_notify (obj, "keyring-info");
+	g_object_notify (obj, "locked");
 	g_object_thaw_notify (obj);
 }
 
@@ -531,3 +721,13 @@ seahorse_gkr_keyring_set_is_default (SeahorseGkrKeyring *self, gboolean is_defau
 	self->pv->is_default = is_default;
 	g_object_notify (G_OBJECT (self), "is-default");
 }
+
+gboolean
+seahorse_gkr_keyring_get_locked (SeahorseGkrKeyring *self)
+{
+	g_return_val_if_fail (SEAHORSE_IS_GKR_KEYRING (self), TRUE);
+
+	if (!self->pv->keyring_info)
+		return TRUE;
+	return gnome_keyring_info_get_is_locked (self->pv->keyring_info);
+}
