@@ -22,10 +22,11 @@
 #ifndef __SEAHORSE_GKR_KEYRING_H__
 #define __SEAHORSE_GKR_KEYRING_H__
 
-#include <glib-object.h>
-
 #include "seahorse-object.h"
 
+#include <glib-object.h>
+#include <gio/gio.h>
+
 #include <gnome-keyring.h>
 
 #define SEAHORSE_TYPE_GKR_KEYRING               (seahorse_gkr_keyring_get_type ())
@@ -52,8 +53,6 @@ GType                seahorse_gkr_keyring_get_type         (void);
 
 SeahorseGkrKeyring*  seahorse_gkr_keyring_new              (const gchar *keyring_name);
 
-void                 seahorse_gkr_keyring_refresh          (SeahorseGkrKeyring *self);
-
 void                 seahorse_gkr_keyring_realize          (SeahorseGkrKeyring *self);
 
 void                 seahorse_gkr_keyring_remove_item      (SeahorseGkrKeyring *self,
@@ -75,13 +74,4 @@ void                 seahorse_gkr_keyring_set_is_default   (SeahorseGkrKeyring *
 
 gboolean             seahorse_gkr_keyring_get_locked       (SeahorseGkrKeyring *self);
 
-void                 seahorse_gkr_keyring_load_async       (SeahorseGkrKeyring *self,
-                                                            GCancellable *cancellable,
-                                                            GAsyncReadyCallback callback,
-                                                            gpointer user_data);
-
-gboolean             seahorse_gkr_keyring_load_finish      (SeahorseGkrKeyring *self,
-                                                            GAsyncResult *result,
-                                                            GError **error);
-
 #endif /* __SEAHORSE_GKR_KEYRING_H__ */
