@@ -22,8 +22,7 @@
 
 #include <glib/gi18n.h>
 
-#include "seahorse-combo-keys.h"
-#include "seahorse-gtkstock.h"
+#include "seahorse-icons.h"
 #include "seahorse-keyserver-control.h"
 #include "seahorse-prefs.h"
 #include "seahorse-registry.h"
@@ -62,16 +61,14 @@ keyserver_cell_edited (GtkCellRendererText *cell, gchar *path, gchar *text,
                        GtkTreeModel *model)
 {
     GtkTreeIter iter;
-    gboolean ret;
-    
+
     if (!seahorse_servers_is_valid_uri (text)) {
         seahorse_util_show_error (NULL, _("Not a valid Key Server address."), 
                                   _("For help contact your system adminstrator or the administrator of the key server." ));
         return;
     }
-    
-    ret = gtk_tree_model_get_iter_from_string (model, &iter, path);
-    g_assert (ret);
+
+    g_warn_if_fail (gtk_tree_model_get_iter_from_string (model, &iter, path));
     gtk_tree_store_set (GTK_TREE_STORE (model), &iter, KEYSERVER_COLUMN, text, -1);
 }
 
@@ -233,20 +230,21 @@ calculate_keyserver_uri (SeahorseWidget *swidget)
     const gchar *host = NULL;
     const gchar *port = NULL;
     GtkWidget *widget;
-    GList *types;
+    gchar **types;
     gint active;
     gchar *uri;
+    guint i;
 
     /* Figure out the scheme */
     widget = GTK_WIDGET (seahorse_widget_get_widget (swidget, "keyserver-type"));
     g_return_val_if_fail (widget != NULL, NULL);
 
 	active = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
-	if (active >= 0) {
-		types = g_object_get_data (G_OBJECT (swidget), "keyserver-types");
-		scheme = (const gchar*)g_list_nth_data (types, active);
-		if (scheme && !scheme[0])
-			scheme = NULL;
+	types = g_object_get_data (G_OBJECT (swidget), "keyserver-types");
+	if (active >= 0 && types) {
+		for (i = 0; types[i] != NULL && i < active; i++);
+		if (i == active && types[active] && types[active][0])
+			scheme = types[active];
 	}
 
     /* The host */
@@ -281,7 +279,7 @@ calculate_keyserver_uri (SeahorseWidget *swidget)
 }
 
 G_MODULE_EXPORT void
-on_prefs_add_keyserver_uri_changed (GtkWidget *button,
+on_prefs_add_keyserver_uri_changed (GtkWidget *unused,
                                     gpointer user_data)
 {
     SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
@@ -340,19 +338,18 @@ on_prefs_keyserver_add_clicked (GtkButton *button,
     /* The list of types */
     types = seahorse_servers_get_types ();
 
+    /* Save these away for later */
+    g_object_set_data_full (G_OBJECT (swidget), "keyserver-types", types,
+                            (GDestroyNotify)g_strfreev);
+
     /* The description for the key server types, plus custom */
     for (i = 0; types[i] != NULL; i++)
         gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget),
                                         seahorse_servers_get_description (types[i]));
     gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget),
                                     _("Custom"));
-    gtk_combo_box_text_remove (GTK_COMBO_BOX_TEXT (widget), 0);
     gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
 
-    /* Save these away for later */
-    g_object_set_data_full (G_OBJECT (swidget), "keyserver-types", types, 
-                            (GDestroyNotify)g_strfreev);
-
     response = gtk_dialog_run (GTK_DIALOG (seahorse_widget_get_toplevel (swidget)));
     if (response == GTK_RESPONSE_ACCEPT) {
         result = calculate_keyserver_uri (swidget);
