@@ -29,7 +29,8 @@
  
 #include "egg-datetime.h"
 
-#include "seahorse-gtkstock.h"
+#include "seahorse-action.h"
+#include "seahorse-icons.h"
 #include "seahorse-registry.h"
 #include "seahorse-passphrase.h"
 #include "seahorse-progress.h"
@@ -37,11 +38,12 @@
 #include "seahorse-widget.h"
 
 #include "seahorse-pgp.h"
+#include "seahorse-pgp-backend.h"
 #include "seahorse-gpgme.h"
 #include "seahorse-gpgme-dialogs.h"
 #include "seahorse-gpgme-key.h"
 #include "seahorse-gpgme-key-op.h"
-#include "seahorse-gpgme-source.h"
+#include "seahorse-gpgme-keyring.h"
 
 /**
  * SECTION:seahorse-gpgme-generate
@@ -49,6 +51,19 @@
  *
  **/
 
+void           on_gpgme_generate_response                    (GtkDialog *dialog,
+                                                              guint response,
+                                                              gpointer user_data);
+
+void           on_gpgme_generate_entry_changed               (GtkEditable *editable,
+                                                              gpointer user_data);
+
+void           on_gpgme_generate_expires_toggled             (GtkToggleButton *button,
+                                                              gpointer user_data);
+
+void           on_gpgme_generate_algorithm_changed           (GtkComboBox *combo,
+                                                              gpointer user_data);
+
 /* --------------------------------------------------------------------------
  * ACTIONS
  */
@@ -64,18 +79,20 @@
 static void
 on_pgp_generate_key (GtkAction *action, gpointer unused)
 {
-	SeahorseSource* sksrc;
-	
+	SeahorseGpgmeKeyring* keyring;
+
 	g_return_if_fail (GTK_IS_ACTION (action));
-	
-	sksrc = seahorse_context_find_source (seahorse_context_instance (), SEAHORSE_PGP_TYPE, SEAHORSE_LOCATION_LOCAL);
-	g_return_if_fail (sksrc != NULL);
-	
-	seahorse_gpgme_generate_show (SEAHORSE_GPGME_SOURCE (sksrc), NULL, NULL, NULL, NULL);
+
+	keyring = seahorse_pgp_backend_get_default_keyring (NULL);
+	g_return_if_fail (keyring != NULL);
+
+	seahorse_gpgme_generate_show (keyring,
+	                              seahorse_action_get_window (action),
+	                              NULL, NULL, NULL);
 }
 
 static const GtkActionEntry ACTION_ENTRIES[] = {
-	{ "pgp-generate-key", SEAHORSE_PGP_STOCK_ICON, N_ ("PGP Key"), "", 
+	{ "pgp-generate-key", GCR_ICON_KEY_PAIR, N_ ("PGP Key"), "",
 	  N_("Used to encrypt email and files"), G_CALLBACK (on_pgp_generate_key) }
 };
 
@@ -161,7 +178,7 @@ on_generate_key_complete (GObject *source,
 {
 	GError *error = NULL;
 
-	if (!seahorse_gpgme_key_op_generate_finish (SEAHORSE_GPGME_SOURCE (source), result, &error))
+	if (!seahorse_gpgme_key_op_generate_finish (SEAHORSE_GPGME_KEYRING (source), result, &error))
 		seahorse_util_handle_error (&error, NULL, _("Couldn't generate PGP key"));
 }
 
@@ -181,7 +198,7 @@ on_generate_key_complete (GObject *source,
  *
  */
 void
-seahorse_gpgme_generate_key (SeahorseGpgmeSource *source,
+seahorse_gpgme_generate_key (SeahorseGpgmeKeyring *keyring,
                              const gchar *name,
                              const gchar *email,
                              const gchar *comment,
@@ -199,7 +216,7 @@ seahorse_gpgme_generate_key (SeahorseGpgmeSource *source,
 	if (gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT) {
 		pass = seahorse_passphrase_prompt_get (dialog);
 		cancellable = g_cancellable_new ();
-		seahorse_gpgme_key_op_generate_async (source, name, email, comment,
+		seahorse_gpgme_key_op_generate_async (keyring, name, email, comment,
 		                                      pass, type, bits, expires,
 		                                      cancellable, on_generate_key_complete,
 		                                      NULL);
@@ -222,9 +239,12 @@ seahorse_gpgme_generate_key (SeahorseGpgmeSource *source,
  *
  */
 G_MODULE_EXPORT void
-on_gpgme_generate_response (GtkDialog *dialog, guint response, SeahorseWidget *swidget)
+on_gpgme_generate_response (GtkDialog *dialog,
+                            guint response,
+                            gpointer user_data)
 {
-    SeahorseGpgmeSource *sksrc;
+    SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
+    SeahorseGpgmeKeyring *keyring;
     GtkWidget *widget;
     gchar *name;
     const gchar *email;
@@ -292,13 +312,13 @@ on_gpgme_generate_response (GtkDialog *dialog, guint response, SeahorseWidget *s
         egg_datetime_get_as_time_t (EGG_DATETIME (widget), &expires);
     }
 
-    sksrc = SEAHORSE_GPGME_SOURCE (g_object_get_data (G_OBJECT (swidget), "source"));
-    g_assert (SEAHORSE_IS_GPGME_SOURCE (sksrc));
+    keyring = SEAHORSE_GPGME_KEYRING (g_object_get_data (G_OBJECT (swidget), "source"));
+    g_assert (SEAHORSE_IS_GPGME_KEYRING (keyring));
 
     /* Less confusing with less on the screen */
     gtk_widget_hide (seahorse_widget_get_toplevel (swidget));
 
-    seahorse_gpgme_generate_key (sksrc, name, email, comment, type, bits, expires);
+    seahorse_gpgme_generate_key (keyring, name, email, comment, type, bits, expires);
 
 
     seahorse_widget_destroy (swidget);
@@ -314,8 +334,10 @@ on_gpgme_generate_response (GtkDialog *dialog, guint response, SeahorseWidget *s
  *
  */
 G_MODULE_EXPORT void
-on_gpgme_generate_entry_changed (GtkEditable *editable, SeahorseWidget *swidget)
+on_gpgme_generate_entry_changed (GtkEditable *editable,
+                                 gpointer user_data)
 {
+    SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
     GtkWidget *widget;
     gchar *name;
 
@@ -339,8 +361,10 @@ on_gpgme_generate_entry_changed (GtkEditable *editable, SeahorseWidget *swidget)
  * Handles the expires toggle button feedback
  */
 G_MODULE_EXPORT void
-on_gpgme_generate_expires_toggled (GtkToggleButton *button, SeahorseWidget *swidget)
+on_gpgme_generate_expires_toggled (GtkToggleButton *button,
+                                   gpointer user_data)
 {
+    SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
     GtkWidget *widget;
     
     widget = get_expiry_date (swidget);
@@ -358,8 +382,10 @@ on_gpgme_generate_expires_toggled (GtkToggleButton *button, SeahorseWidget *swid
  *
  */
 G_MODULE_EXPORT void
-on_gpgme_generate_algorithm_changed (GtkComboBox *combo, SeahorseWidget *swidget)
+on_gpgme_generate_algorithm_changed (GtkComboBox *combo,
+                                     gpointer user_data)
 {
+    SeahorseWidget *swidget = SEAHORSE_WIDGET (user_data);
     GtkWidget *widget;
     gint sel;
     
@@ -382,7 +408,7 @@ on_gpgme_generate_algorithm_changed (GtkComboBox *combo, SeahorseWidget *swidget
 
 /**
  * seahorse_gpgme_generate_show:
- * @sksrc: the gpgme source
+ * @keyring: the gpgme source
  * @parent: the parent window
  * @name: The user name, can be NULL if not available
  * @email: The user's email address, can be NULL if not available
@@ -392,7 +418,11 @@ on_gpgme_generate_algorithm_changed (GtkComboBox *combo, SeahorseWidget *swidget
  *
  */
 void
-seahorse_gpgme_generate_show (SeahorseGpgmeSource *sksrc, GtkWindow *parent, const gchar * name, const gchar *email, const gchar *comment)
+seahorse_gpgme_generate_show (SeahorseGpgmeKeyring *keyring,
+                              GtkWindow *parent,
+                              const gchar * name,
+                              const gchar *email,
+                              const gchar *comment)
 {
     SeahorseWidget *swidget;
     GtkWidget *widget, *datetime;
@@ -425,12 +455,7 @@ seahorse_gpgme_generate_show (SeahorseGpgmeSource *sksrc, GtkWindow *parent, con
         g_return_if_fail (widget != NULL);
         gtk_entry_set_text(GTK_ENTRY(widget),comment);
     }
-    
-    widget = seahorse_widget_get_widget (swidget, "pgp-image");
-    g_return_if_fail (widget != NULL);
-    gtk_image_set_from_stock (GTK_IMAGE (widget), SEAHORSE_STOCK_SECRET, 
-                              GTK_ICON_SIZE_DIALOG);
-    
+
     /* The algoritms */
     widget = seahorse_widget_get_widget (swidget, "algorithm-choice");
     g_return_if_fail (widget != NULL);
@@ -450,9 +475,8 @@ seahorse_gpgme_generate_show (SeahorseGpgmeSource *sksrc, GtkWindow *parent, con
     gtk_box_pack_start (GTK_BOX (widget), datetime, TRUE, TRUE, 0);
     gtk_widget_set_sensitive (datetime, FALSE);
     gtk_widget_show_all (widget);
-    
-    g_object_ref (sksrc);
-    g_object_set_data_full (G_OBJECT (swidget), "source", sksrc, g_object_unref);
-	
+
+    g_object_set_data_full (G_OBJECT (swidget), "source", g_object_ref (keyring), g_object_unref);
+
     on_gpgme_generate_entry_changed (NULL, swidget);
 }
